/**
 * ContentsParser - parses the contents file
 * @author Peter Tribble
 * @version 1.0
 *
 */

package uk.co.petertribble.pkgview;

import java.util.*;
import java.io.*;

/**
 * We parse the Solaris contents file and create two hashes.
 *
 * The first hash is by file, or equivalently by line.
 * The key is the filename, the value is a ContentsFileDetail
 * which stores the metadata and the list of packages that contain
 * this file.
 *
 * The second hash is by package. The key is the package name,
 * and the value is a List of ContentsFileDetail's.
 */
public class ContentsParser {

    private static ContentsParser cpinstance;

    private String contents;
    private Map <String, ContentsFileDetail> fileHash;
    private Map <String, ContentsPackage> pkgHash;

    private static final String CONTENTS_FILE = "/var/sadm/install/contents";

    /**
     * Parse the default contents file.
     */
    private ContentsParser() {
	this(CONTENTS_FILE);
    }

    /**
     * Parse a contents file
     */
    private ContentsParser(String contents) {
	fileHash = new HashMap <String, ContentsFileDetail> ();
	pkgHash = new HashMap <String, ContentsPackage> ();
	this.contents = contents;
	parse();
    }

    public static synchronized ContentsParser getInstance() {
	if (cpinstance == null) {
	    cpinstance = new ContentsParser();
	}
	return cpinstance;
    }

    /*
     * Oddly, using this version is significantly slower, although
     * it does consume rather less memory. And timing of the actual
     * reading of the file indicates that this is a lot faster, so
     * I'm not sure why the overall time gets worse.
     */
    private void parse() {
	try {
	    BufferedReader in
		= new BufferedReader(new FileReader(contents));
	    String s = null;
	    while ((s = in.readLine()) != null) {
		ContentsFileDetail cfd = new ContentsFileDetail(s);
		fileHash.put(cfd.getName(), cfd);
		for (String pkgname : cfd.getPackageNames()) {
		    ContentsPackage cp = pkgHash.get(pkgname);
		    if (cp == null) {
			cp = new ContentsPackage();
			pkgHash.put(pkgname, cp);
		    }
		    cp.addFile(cfd);
		}
	    }
	} catch (IOException ioe) {}
    }

    public ContentsFileDetail getFileDetail(String s) {
	return fileHash.get(s);
    }

    public ContentsPackage getPackage(String pkgname) {
	return pkgHash.get(pkgname);
    }

    public static void main(String[] args) {
	new ContentsParser();
    }

}
