/**
 * InstalledFilesPanel
 * @author Peter Tribble
 * @version 1.0
 *
 */

package uk.co.petertribble.pkgview;

import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import java.awt.*;
import java.io.File;

/**
 * Show a tree view of the local filesystem. Selecting a file causes
 * information about the software package(s) associated with that file
 * to appear in an area to the right of the tree.
 */
public class InstalledFilesPanel extends JPanel {

    private JPanel jptree;
    private PackageInformationPanel pip;

    private ClusterToc ctoc;
    private ContentsParser cp;

    public InstalledFilesPanel(ClusterToc ctoc) {
	this.ctoc = ctoc;

	setLayout(new BorderLayout());

	jptree = new JPanel(new BorderLayout());
	pip = new PackageInformationPanel(true, false, ctoc.exists(), false,
					false);

	JSplitPane psplit = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
		jptree, pip);
	psplit.setOneTouchExpandable(true);
	psplit.setDividerLocation(180);
	add(psplit);

	addFileTree();
    }

    private void addFileTree() {
	final ContentsFileTree cft = new ContentsFileTree();
	cft.addTreeSelectionListener(new TreeSelectionListener() {
	    public void valueChanged(TreeSelectionEvent e) {
		DefaultMutableTreeNode node = (DefaultMutableTreeNode)
		    cft.getLastSelectedPathComponent();

		if (node != null) {
		    setFileInfo((File) node.getUserObject());
		}
	    }
	});
	jptree.add(new JScrollPane(cft));
    }

    private void setFileInfo(File f) {
	if (cp == null) {
	    pip.setInfoText("Package information not available.");
	    pip.setClusterText("");
	    return;
	}
	ContentsFileDetail cfd = cp.getFileDetail(f.toString());
	if (cfd == null) {
	    pip.setInfoText("Not a member of any package.");
	    pip.setClusterText("");
	} else {
	    pip.setInfoText(fileDetailTable(cfd));
	    setFileMembership(cfd);
	}
    }

    private String fileDetailTable(ContentsFileDetail cfd) {
	StringBuilder sb = new StringBuilder();
	sb.append("<table width=\"100%\">");
	sb.append("<tr bgcolor=\"#eeeeee\"><th colspan=\"2\">");
	sb.append("Path name: ").append(cfd.getName());
	sb.append("</th></tr>\n");
	sb.append("<tr><td>");
	sb.append("File Type:");
	sb.append("</td><td>");
	sb.append(cfd.getDescriptiveType());
	sb.append("</td></tr>");
	if (cfd.isLink()) {
	    sb.append("<tr><td>");
	    sb.append("Link target:");
	    sb.append("</td><td>");
	    sb.append(cfd.target);
	    sb.append("</td></tr>");
	} else {
	    sb.append("<tr><td>");
	    sb.append("Owner:");
	    sb.append("</td><td>");
	    sb.append(cfd.owner);
	    sb.append("</td></tr>");
	    sb.append("<tr><td>");
	    sb.append("Group owner:");
	    sb.append("</td><td>");
	    sb.append(cfd.group);
	    sb.append("</td></tr>");
	    sb.append("<tr><td>");
	    sb.append("Permissions:");
	    sb.append("</td><td>");
	    sb.append(cfd.mode);
	    sb.append("</td></tr>");
	    if (cfd.isRegular()) {
		sb.append("<tr><td>");
		sb.append("Size:");
		sb.append("</td><td>");
		sb.append(cfd.size);
		sb.append("</td></tr>");
	    }
	}
	sb.append("</table>\n");
	sb.append("<table width=\"100%\">");
	sb.append("<tr bgcolor=\"#eeeeee\"><th>");
	sb.append("This file is a member of the following packages");
	sb.append("</th></tr>\n");
	for (String pname : cfd.getPackageNames()) {
	    sb.append("<tr><td>");
	    sb.append(pname);
	    sb.append("</td></tr>\n");
	}
	sb.append("</table>\n");
	return sb.toString();
    }

    /**
     * Display which clusters and metaclusters a given file is in.
     */
    private void setFileMembership(ContentsFileDetail cfd) {
	StringBuilder sb = new StringBuilder();
	sb.append("<table width=\"100%\">");
	sb.append("<tr bgcolor=\"#eeeeee\"><th colspan=\"2\">");
	sb.append("This file is part of the following clusters:");
	sb.append("</th></tr>\n");
	for (PackageCluster pc : ctoc.containingClusters(cfd.getPackages())) {
	    sb.append("<tr><td>").append(pc).append("</td><td>");
	    sb.append(pc.getName()).append("</td></tr>\n");
	}
	sb.append("<tr bgcolor=\"#eeeeee\"><th colspan=\"2\">");
	sb.append("This file is part of the following Metaclusters:");
	sb.append("</th></tr>\n");
	for (MetaCluster mc : ctoc.containingMetaClusters(cfd.getPackages())) {
	    sb.append("<tr><td>").append(mc).append("</td><td>");
	    sb.append(mc.getName()).append("</td></tr>\n");
	}
	sb.append("</table>\n");
	pip.setClusterText(sb.toString());
    }

    public void showDetailedView() {
	cp = ContentsParser.getInstance();
    }
}
