/**
 * PkgList - shows available Solaris packages in a JList
 * @author Peter Tribble
 * @version 1.0
 *
 */

package uk.co.petertribble.pkgview;

import java.io.*;
import java.util.*;

public class PkgList {

    private List <SolarisPackage> pkglist;
    private Map <String, SolarisPackage> pkgMap;
    private Map <String, Set <SolarisPackage>> revDependencies = null;
    private Set <SolarisPackage> revSet = null;

    /**
     * Create a package list
     */
    public PkgList() {
	this("/var/sadm/pkg");
    }

    /**
     * Create a package list
     *
     * Packages are in directories, so ignore files
     * and anything hidden (starting with a dot) and
     * also the locale directory
     */
    public PkgList(String pkgroot) {
	pkgMap = new HashMap <String, SolarisPackage> ();
	pkglist = new ArrayList <SolarisPackage> ();

	File pkgrootf = new File(pkgroot);
	File[] fdirs = pkgrootf.listFiles();
	Arrays.sort(fdirs);

	// FIXME check for a pkginfo file
	for (int k = 0; k < fdirs.length; k++) {
	    if (fdirs[k].isDirectory() &&
		!fdirs[k].isHidden() &&
		!fdirs[k].getName().equals("locale")) {
		SolarisPackage sp = new SolarisPackage(
			pkgroot, fdirs[k].getName());
		pkglist.add(sp);
		pkgMap.put(sp.getName(), sp);
	    }
	}
    }

    public List <SolarisPackage> getPackages() {
	return pkglist;
    }

    public SolarisPackage getPackage(String name) {
	return pkgMap.get(name);
    }

    public Set <SolarisPackage> getDependantSet(String pkg) {
	if (revDependencies == null) {
	    return null;
	}
	return revDependencies.get(pkg);
    }

    /**
     * Return a HTML table showing the dependants of this package.
     */
    public String revDeps(String pkgname) {
	return revDeps(pkgname, false);
    }

    /**
     * Return a HTML table showing the dependants of this package and,
     * optionally, whether they are currently selected.
     */
    public String revDeps(String pkgname, boolean showselected) {
	revSet = getDependantSet(pkgname);
	if (revSet == null) {
	    return "";
	}
	StringBuilder sb = new StringBuilder();
	sb.append("<table width=\"100%\"><tr bgcolor=\"#eeeeee\">");
	sb.append("<th>Depending on this package:</th></tr>\n");
	for (SolarisPackage pkg : revSet) {
	    sb.append("<tr><td>").append(pkg.getName());
	    if (showselected && pkg.getSelected()) {
		sb.append(" (selected)");
	    }
	    sb.append("</td></tr>\n");
	}
	sb.append("</table>\n");
	return sb.toString();
    }

    /**
     * Create a reverse dependency tree. Pull the dependencies out
     * and populate another Map.
     */
    public void createRevDependencies() {
	revDependencies = new HashMap <String, Set <SolarisPackage>> ();
	for (SolarisPackage pkg : pkglist) {
	    Map <String, String> hdep = pkg.dependMap();
	    for (String pkgdep : hdep.keySet()) {
		String dtype = hdep.get(pkgdep);
		if (dtype.equals("P")) {
		    revSet = revDependencies.get(pkgdep);
		    if (revSet == null) {
			revSet = new HashSet <SolarisPackage> ();
			revDependencies.put(pkgdep, revSet);
		    }
		    revSet.add(pkg);
		}
	    }
	}
	/*
	 * Now we've built the tree, tell the packages what their
	 * dependants are.
	 */
	for (SolarisPackage pkg : pkglist) {
	    pkg.setDependantSet(revDependencies.get(pkg.getName()));
	}
    }
}
