/**
 * PkgUtils - shows available Solaris packages
 * @author Peter Tribble
 * @version 1.0
 *
 */

package uk.co.petertribble.pkgview;

import java.util.*;
import java.text.DecimalFormat;

public class PkgUtils {

    /**
     * Returns the pkginfo file as a html table
     */
    static public String infoTable(SolarisPackage pkg) {
	StringBuilder sb = new StringBuilder();
	// FIXME infoHash is a useless API
	Map <String, String> infomap = pkg.infoMap();
	// clean out the junk
	infomap.remove("#FASPACD");
	infomap.remove("PKG");
	infomap.remove("MAXINST");
	infomap.remove("PKGINST");
	infomap.remove("PKGSAV");
	infomap.remove("OAMBASE");
	infomap.remove("ARCH");
	infomap.remove("TZ");
	infomap.remove("PATH");
	infomap.remove("UPDATE");
	infomap.remove("ISTATES");
	infomap.remove("LANG");
	Iterator itr = infomap.keySet().iterator();
	while (itr.hasNext()) {
	    String s = (String) itr.next();
	    if (s.startsWith("LC_") || s.startsWith("PATCH_")
		   || s.startsWith("SUNW_") || s.startsWith("ACTIVE_")
		   || s.startsWith("SCRIPTS_") || s.startsWith("PKG_")) {
		itr.remove();
	    }
	}
	sb.append("<table width=\"100%\"><tr bgcolor=\"#eeeeee\">");
	sb.append("<th>").append(PkgResources.getString("PKGUTILS.PROPERTY"));
	sb.append("</th><th>").append(PkgResources.getString("PKGUTILS.VALUE"));
	sb.append("</th></tr>\n");
	for (String s : infomap.keySet()) {
	    String v = infomap.get(s);
	    if (!v.equals("")) {
		sb.append("<tr><td>").append(s).append("</td><td>");
		sb.append(v).append("</td></tr>\n");
	    }
	}
	sb.append("</table>\n");
	return sb.toString();
    }

    static public String dependencyTable(SolarisPackage pkg, PkgList plist) {
	return dependencyTable(pkg, plist, false);
    }

    static public String dependencyTable(SolarisPackage pkg, PkgList plist,
					boolean showselected) {
	return dependencyTable(pkg.dependMap(), plist, showselected);
    }

    static public String dependencyTable(PackageCluster pc, PkgList plist) {
	return dependencyTable(pc, plist, false);
    }

    static public String dependencyTable(PackageCluster pc, PkgList plist,
					boolean showselected) {
	return dependencyTable(pc.dependMap(), plist, showselected);
    }

    /**
     * Common dependency tree code
     */
    static private String dependencyTable(Map <String, String> h, PkgList plist,
					boolean showselected) {
	StringBuilder sb = new StringBuilder();
	sb.append("<table width=\"100%\"><tr bgcolor=\"#eeeeee\">");
	sb.append("<th>").append(PkgResources.getString("PKGUTILS.PACKAGE"));
	sb.append("</th><th>");
	sb.append(PkgResources.getString("PKGUTILS.DEPENDENCY"));
	sb.append("</th></tr>\n");
	for (String s : h.keySet()) {
	    String d = h.get(s);
	    sb.append("<tr><td>").append(s);
	    if (showselected) {
		SolarisPackage pkg = plist.getPackage(s);
		if ((pkg != null) && (!pkg.getSelected())) {
		    sb.append(" (unselected)");
		}
	    }
	    sb.append("</td><td>");
	    if (d.equals("P")) {
		sb.append(PkgResources.getString("PKGUTILS.PREREQ"));
	    } else if (d.equals("I")) {
		sb.append(PkgResources.getString("PKGUTILS.INCOMP"));
	    } else if (d.equals("R")) {
		sb.append(PkgResources.getString("PKGUTILS.REQ"));
	    } else {
		sb.append(PkgResources.getString("PKGUTILS.UNKNOWN"));
	    }
	    sb.append("</td></tr>\n");
	}
	sb.append("</table>\n");
	return sb.toString();
    }

    static public String detailTable(PackageCluster pc, ContentsParser cp) {
	StringBuilder sb = new StringBuilder();
	if (cp != null) {
	    ContentsCluster cc = new ContentsCluster(pc);

	    sb.append(doDetailTable(cc.numEntries(), cc.numShared(),
		cc.numHardLinks(), cc.numDirectories(), cc.spaceUsed()));
	}
	return sb.toString();
    }

    static public String detailTable(SolarisPackage pkg, ContentsParser cp) {
	StringBuilder sb = new StringBuilder();
	if (cp != null) {
	    ContentsPackage cpp = cp.getPackage(pkg.getName());
	    if (cpp == null) {
		return (String) null;
	    }

	    sb.append(doDetailTable(cpp.numEntries(), cpp.numShared(),
		cpp.numHardLinks(), cpp.numDirectories(), cpp.spaceUsed()));
	}
	return sb.toString();
    }

    static private String doDetailTable(long instpath, long shpath,
					long lnfiles, long dirs, long space) {
	StringBuilder sb = new StringBuilder();

	sb.append("<table width=\"100%\"><tr bgcolor=\"#eeeeee\">");
	sb.append("<th colspan=\"2\">");
	sb.append(PkgResources.getString("PKGUTILS.DETAILS"));
	sb.append("</th></tr>\n");

	sb.append("<tr><td>");
	sb.append(PkgResources.getString("PKGUTILS.INSTPATH"));
	sb.append("</td><td>");
	sb.append(instpath).append("</td></tr>\n");

	sb.append("<tr><td>");
	sb.append(PkgResources.getString("PKGUTILS.SHPATH"));
	sb.append("</td><td>");
	sb.append(shpath).append("</td></tr>\n");

	sb.append("<tr><td>");
	sb.append(PkgResources.getString("PKGUTILS.LNFILES"));
	sb.append("</td><td>");
	sb.append(lnfiles).append("</td></tr>\n");

	sb.append("<tr><td>");
	sb.append(PkgResources.getString("PKGUTILS.DIRS"));
	sb.append("</td><td>");
	sb.append(dirs).append("</td></tr>\n");

	sb.append("<tr><td>");
	sb.append(PkgResources.getString("PKGUTILS.SPC"));
	sb.append("</td><td>");
	sb.append(niceSpaceUsed(space));
	sb.append("</td></tr>\n");

	sb.append("</table>\n");

	return sb.toString();
    }

    static private String niceSpaceUsed(long space) {
        DecimalFormat df = new DecimalFormat("##0.0#");
	StringBuilder sb = new StringBuilder();
	double dspace = (double) space;
	int iscale = 0;
	while ((dspace > 1024.0) && (iscale < 3)) {
	    iscale++;
	    dspace = dspace/1024.0;
	}
	sb.append(df.format(dspace));
	sb.append(" ");
	switch (iscale) {
	    case 0:
		sb.append(PkgResources.getString("PKGUTILS.BYTES"));
		break;
	    case 1:
		sb.append(PkgResources.getString("PKGUTILS.KBYTES"));
		break;
	    case 2:
		sb.append(PkgResources.getString("PKGUTILS.MBYTES"));
		break;
	    case 3:
		sb.append(PkgResources.getString("PKGUTILS.GBYTES"));
		break;
	}
	return sb.toString();
    }

    static public String clusterMembership(SolarisPackage pkg,
		ClusterToc ctoc) {
	StringBuilder sb = new StringBuilder();
	sb.append("<table width=\"100%\">");
	sb.append("<tr bgcolor=\"#eeeeee\"><th colspan=\"2\">");
	sb.append(PkgResources.getString("PKGUTILS.CL"));
	sb.append("</th></tr>\n");
	for (PackageCluster pclust : ctoc.containingClusters(pkg)) {
	    sb.append("<tr><td>").append(pclust).append("</td><td>");
	    sb.append(pclust.getName()).append("</td></tr>\n");
	}
	sb.append("<tr bgcolor=\"#eeeeee\"><th colspan=\"2\">");
	sb.append(PkgResources.getString("PKGUTILS.MC"));
	sb.append("</th></tr>\n");
	for (MetaCluster mclust : ctoc.containingMetaClusters(pkg)) {
	    sb.append("<tr><td>").append(mclust).append("</td><td>");
	    sb.append(mclust.getName()).append("</td></tr>\n");
	}
	sb.append("</table>\n");
	return sb.toString();
    }
}
