/**
 * SolarisPackage - describes a Solaris package
 * @author Peter Tribble
 * @version 1.0
 *
 */

package uk.co.petertribble.pkgview;

import java.util.*;
import java.io.File;
import uk.co.petertribble.jumble.*;

public class SolarisPackage {

    private File pkgrootf;

    private String pkgname;
    // corresponds to the NAME field
    private String name;
    private Map <String, String> infomap = null;
    private Map <String, String> mdepend = null;
    private Set <SolarisPackage> depSet = null;

    private List <String> categories;
    // corresponds to the VERSION field
    private String version;
    // corresponds to the DESC field
    private String description;

    private boolean selected = false;
    private boolean mustbeselected = false;

    /**
     * Create a Solaris package container
     */
    public SolarisPackage(String pkgname) {
	this("/var/sadm/pkg", pkgname);
    }

    /**
     * Create a Solaris package container
     */
    public SolarisPackage(String pkgroot, String pkgname) {
	pkgrootf = new File(pkgroot);
	this.pkgname = pkgname;
    }

    /**
     * The name of this package
     */
    public String getName() {
	return pkgname;
    }

    /**
     * Set the descriptive name of this package
     */
    public void setName(String name) {
	this.name = name;
    }

    /**
     * The name of this package
     */
    public String toString() {
	return pkgname;
    }

    /**
     * Set the long name, or description, of this package.
     */
    public void setDescription(String description) {
	this.description = description;
    }

    /**
     * The long name, or description of this package
     */
    public String getDescription() {
	String s;
	s = description;
	if (s == null) {
	    s = getInfoItem("NAME");
	}
	if (s == null) {
	    s = getInfoItem("DESC");
	}
	if (s == null) {
	    s = pkgname;
	}
	return s;
    }

    /**
     * Set the version of this package.
     */
    public void setVersion(String version) {
	this.version = version;
    }

    /**
     * Get the version of this package
     */
    public String getVersion() {
	String s;
	s = version;
	if (s == null) {
	    s = getInfoItem("VERSION");
	}
	return s;
    }

    /**
     * Add a dependency
     */
    public void addDependency(String pkg, String type) {
	if (mdepend == null) {
	    mdepend = new HashMap <String, String> ();
	}
	mdepend.put(pkg, type);
    }

    /**
     * Add a list of categories.
     *
     * @param categories A string containing a list of categories separated
     * by commas.
     */
    public void setCategories(String categories) {
	String[] sc = categories.split(",");
	for (int i = 0; i < sc.length; i++) {
	    addCategory(sc[i]);
	}
    }

    /**
     * Add a category.
     */
    public void addCategory(String category) {
	if (categories == null) {
	    categories = new ArrayList <String> ();
	}
	categories.add(category);
    }

    /**
     * Parse the depend file.
     *
     * FIXME it's incompatible with addDependency - is this a problem
     */
    public Map <String, String> dependMap() {
	if (mdepend == null) {
	    mdepend = new HashMap <String, String> ();
	    StringTokenizer st = new StringTokenizer(getDepend(), "\n");
	    while (st.hasMoreTokens()) {
		String[] ds = st.nextToken().split("\\s+", 3);
		// Must have at least 2 words
		if (ds.length > 1) {
		    if (ds[0].equals("P") || ds[0].equals("I") ||
						ds[0].equals("R")) {
			mdepend.put(ds[1], ds[0]);
		    }
		}
	    }
	}
	return mdepend;
    }

    /**
     * Return the Set of package names that this package is dependent on.
     */
    public Set <String> getDependencySet() {
	Set <String> deps = new HashSet <String> ();
	Map <String, String> m = dependMap();
	for (String s : m.keySet()) {
	    if (m.get(s).equals("P")) {
		deps.add(s);
	    }
	}
	return deps;
    }

    /**
     * Define the set of packages that are dependant on this package.
     */
    public void setDependantSet(Set <SolarisPackage> depSet) {
	this.depSet = depSet;
    }

    /**
     * Return the set of packages that are dependant on this package.
     */
    public Set <SolarisPackage> getDependantSet() {
	return depSet;
    }

    /**
     * Return the Set of packages that are dependant on this package
     * and are selected.
     */
    public Set <SolarisPackage> getSelectedDependantSet() {
	Set <SolarisPackage> seldepSet = new HashSet <SolarisPackage> ();
	if (depSet != null) {
	    for (SolarisPackage pkg : getDependantSet()) {
		if (pkg.getSelected()) {
		    seldepSet.add(pkg);
		}
	    }
	}
	return seldepSet;
    }

    /**
     * Parse the pkginfo file
     */
    public Map <String, String> infoMap() {
	if (infomap == null) {
	    infomap = JumbleUtils.stringToPropMap(getInfo(), "\n");
	}
	// defensive copy
	return new HashMap <String, String> (infomap);
    }

    /**
     * Get the specified property from the pkginfo file
     */
    public String getInfoItem(String s) {
	return infoMap().get(s);
    }

    /**
     * Returns whether this package exists, by seeing whether the
     * directory corresponding to its name exists.
     */
    public boolean exists() {
	return new File(pkgrootf, pkgname).exists();
    }

    /**
     * Returns the pkginfo file associated with this package as a String
     */
    private String getInfo() {
	return JumbleFile.getStringContents(
		new File(pkgrootf, pkgname+"/pkginfo"));
    }

    /**
     * Returns the depend file associated with this package as a String
     */
    public String getDepend() {
	return JumbleFile.getStringContents(
		new File(pkgrootf, pkgname+"/install/depend"));
    }

    /**
     * Sets whether this package is selected. If the package must be
     * selected, the selected status will be set to true.
     */
    public void setSelected(boolean selected) {
	if (mustbeselected) {
	    this.selected = true;
	} else {
	    this.selected = selected;
	}
    }

    /**
     * Gets whether this package is selected
     */
    public boolean getSelected() {
	return selected;
    }

    /**
     * Sets whether this package must be selected.
     */
    public void forceSelected() {
	mustbeselected = true;
    }

    /**
     * Gets whether this package must be selected. Packages and
     * clusters in the SUNWCmreq metacluster must be selected.
     */
    public boolean mustBeSelected() {
	return mustbeselected;
    }
}
