/**
 * DiskInfoPanel - shows Disk status
 * @author Peter Tribble
 * @version 1.0
 *
 */

package uk.co.petertribble.solview.explorer;

import javax.swing.*;
import uk.co.petertribble.jkstat.api.*;
import uk.co.petertribble.jkstat.gui.AccessoryIOPanel;
import uk.co.petertribble.jkstat.gui.KstatTable;
import uk.co.petertribble.solview.InfoCommand;

public class DiskInfoPanel extends InfoPanel {

    private SysItem hi;
    private AccessoryIOPanel acp;
    private JKstat jkstat;
    private Kstat ks;
    private KstatTable kt;

    /**
     * Display a Disk information panel.
     */
    public DiskInfoPanel(SysItem hi, JKstat jkstat) {
	super(hi);
	this.hi = hi;
	this.jkstat = jkstat;
	ks = (Kstat) hi.getAttribute("kstat");

	display();
	validate();
    }

    public void stopLoop() {
	if (acp != null) {
	    acp.stopLoop();
	}
	if (kt != null) {
	    kt.stopLoop();
	}
    }

    /*
     * Display an information panel appropriate to the Disk item that is
     * selected.
     */
    private void display() {
	switch (hi.getType()) {
	    case SysItem.DISK:
		displayDisk();
		break;
	    case SysItem.DISK_PARTITION:
		displayPartition();
		break;
	    case SysItem.DISK_META_CONTAINER:
	    case SysItem.DISK_METADEVICE:
		displayMetadevice();
		break;
	    case SysItem.DISK_CONTAINER:
		displaySummary();
		break;
	}
    }

    /*
     * Top level summary. This ought to be a variant on iostat -E.
     */
    private void displaySummary() {
	jvp.add(new JLabel("Disk Summary"));
	jvp.add(new JLabel("System contains " + "FIXME" + " disks"));
    }

    /*
     * An svm metadevice
     */
    private void displayMetadevice() {
	if (ks == null) {
	    InfoCommand ic = new InfoCommand("MD", "/usr/sbin/metastat");
	    if (ic.exists()) {
		jvp.add(new JLabel("Metadevice status"));
		addText(ic);
	    }
	} else {
	    jvp.add(new JLabel("Details of SVM metadevice " + ks.getName()));
	    // see what we can grab from the config
	    SVMconfig svmconfig = (SVMconfig) hi.getAttribute("config");
	    if (svmconfig != null) {
		MetaDevice md = svmconfig.getDevice(ks.getName().substring(1));
		if (md != null) {
		    MetaDevice mdp = md.getParent();
		    if (mdp != null) {
			jvp.add(new JLabel(md.getTypeAsString() + " " +
				md.getName() + " is part of " +
				mdp.getTypeAsString() + " " + mdp.getName()));
		    }
		}
	    }
	    addAccessory();
	    // the metadevice names strip the leading m, so the kstat md0
	    // corresponds to metadevice d0
	    addText(new InfoCommand("MD", "/usr/sbin/metastat",
						ks.getName().substring(1)));
	}
    }

    /*
     * A disk device
     */
    private void displayDisk() {
	if (ks != null) {
	    jvp.add(new JLabel("Details of Disk " + ks.getName()));
	    addAccessory();
	    addDiskInfo();
	}
    }

    /*
     * A disk partition (or is that slice?)
     */
    private void displayPartition() {
	if (ks != null) {
	    String[] ds = ks.getName().split(",", 2);
	    jvp.add(new JLabel("Details of partition " + ds[1] + " on disk "
						+ ds[0]));
	    addAccessory();
	}
    }

    /*
     * Display information like iostat -E.
     * NOTE: we've already checked for ks being non-null above
     */
    private void addDiskInfo() {
	Kstat ksd = jkstat.getKstat(ks.getModule() + "err", ks.getInst(),
				    ks.getName() + ",err");
	if (ksd != null) {
	    kt = new KstatTable(ks.getModule() + "err", ks.getInstance(),
				ks.getName() + ",err", -1, jkstat);
	    jvp.add(new JScrollPane(kt));
	}
	// wouldn't life be simple if names were consistent?
	ksd = jkstat.getKstat(ks.getModule() + "error", ks.getInst(),
				    ks.getName() + ",error");
	if (ksd != null) {
	    kt = new KstatTable(ks.getModule() + "error", ks.getInstance(),
				ks.getName() + ",error", -1, jkstat);
	    jvp.add(new JScrollPane(kt));
	}
    }

    /*
     * Add an accessory if we can.
     * NOTE: we've already checked for ks being non-null above
     */
    private void addAccessory() {
	acp = new AccessoryIOPanel(ks.getModule(),
				   ks.getInst(), ks.getName(), 5, jkstat);
	jvp.add(acp);
    }

    /*
     * Run a command that generates text
     */
    private void addText(InfoCommand ic) {
	if (ic.exists()) {
	    jvp.add(new JScrollPane(new SysCmdPanel(ic)));
	}
    }
}
