/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

package uk.co.petertribble.solview.explorer;

import javax.swing.*;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.*;
import uk.co.petertribble.jkstat.api.*;
import uk.co.petertribble.jkstat.gui.*;
import uk.co.petertribble.solview.SolViewResources;
import java.util.*;
import uk.co.petertribble.jingle.SpringUtilities;

public class JCpuState implements ActionListener {

    private JKstat jkstat;

    private JFrame jf;
    private JPanel mainPanel;

    private ProcessorTree proctree;
    private Set <Kstat> kstats;
    private int ncpus;
    private int naggr;
    private int ncpu;

    private boolean standalone;

    private JMenuItem exitItem;
    private JMenuItem[] aboutCpuItem;
    private JMenuItem[] extendedCpuItem;
    private JMenuItem[] chartCpuItem;
    private String[] cpuID;
    private java.util.List <AccessoryCpuPanel> acplist;
    private java.util.List <AggregateCpuPanel> agplist;

    private int chipwidth = 200;
    private int chipheight = 20;
    private int corewidth = 200;
    private int coreheight = 20;
    private int threadwidth = 200;
    private int threadheight = 20;
    private int midheight = 15;
    private int smallheight = 10;
    private Dimension dchip;
    private Dimension dcore;
    private Dimension dthread;
    // localised via properties
    private String chipText;
    private String coreText;
    private String cpuText;
    private String cpuLabel;
    private String cpuChartLabel;
    private int fontScale;

    // for optional menu item
    private boolean showChips = true;
    private boolean showCores = true;
    private boolean showThreads = true;
    private JCheckBoxMenuItem showChipItem;
    private JCheckBoxMenuItem showCoreItem;
    private JCheckBoxMenuItem showThreadItem;

    private SolViewResources svr;

    public JCpuState() {
	this(new JKstat(), new String[0], true);
    }

    public JCpuState(String[] args) {
	this(new JKstat(), args, true);
    }

    public JCpuState(JKstat jkstat, boolean standalone) {
	this(jkstat, new String[0], standalone);
    }

    public JCpuState(JKstat jkstat, String[] args, boolean standalone) {
	jf = new JFrame("jcpustate");
	jf.addWindowListener(new winExit());
	this.jkstat = jkstat;
	this.standalone = standalone;

	svr = SolViewResources.getInstance();

	KstatFilter ksf = new KstatFilter(jkstat);
	ksf.setFilterClass("misc");
	ksf.addFilter("cpu_stat:::");

	kstats = ksf.getKstats();

	ncpus = kstats.size();
	aboutCpuItem = new JMenuItem[ncpus];
	extendedCpuItem = new JMenuItem[ncpus];
	chartCpuItem = new JMenuItem[ncpus];
	cpuID = new String[ncpus];
	acplist = new ArrayList <AccessoryCpuPanel> ();
	agplist = new ArrayList <AggregateCpuPanel> ();

	// create main display panel
	SpringLayout layout = new SpringLayout();
	mainPanel = new JPanel(layout);
	jf.setContentPane(mainPanel);

	// get the logical structure of the cpus
	proctree = new ProcessorTree(jkstat);

	/*
	 * Parse arguments. This has to be done after producing the
	 * ProcessorTree so that we know which arguments are valid.
	 */
	parseArgs(args);

	// create localized labels
	chipText = svr.getString("CPUSTATE.CHIP") + " ";
	coreText = svr.getString("CPUSTATE.CORE") + " ";
	cpuText = svr.getString("CPUSTATE.CPU") + " ";

	// change the labels depending on structure
	if (proctree.isMulticore()) {
	    cpuText = svr.getString("CPUSTATE.CORE") + " ";
	}
	if (proctree.isThreaded()) {
	    cpuText = svr.getString("CPUSTATE.THREAD") + " ";
	    fontScale = -2;
	}
	cpuLabel = svr.getString("CPUSTATE.ABOUT.TEXT") + " " + cpuText;
	cpuChartLabel = svr.getString("CPUSTATE.CHART.TEXT") + " " + cpuText;

	setDimensions();

	populate();

	addMenus();

	jf.pack();
	jf.setVisible(true);
    }

    private void setDimensions() {
	if (proctree.isMulticore()) {
	    // reduce the size of the subcomponents
	    coreheight = midheight;
	    threadheight = midheight;
	    if (proctree.isThreaded()) {
		// further reduce the thread sizes
		threadheight = smallheight;
	    }
	} else if (proctree.isThreaded()) {
	    // single core cpus
	    // reduce the size of the subcomponents
	    threadheight = midheight;
	}
	dchip = new Dimension(chipwidth, chipheight);
	dcore = new Dimension(corewidth, coreheight);
	dthread = new Dimension(threadwidth, threadheight);
    }

    private void populate() {
	if (proctree.isMulticore()) {
	    // iterate over chips, showing aggregates over cores
	    for (Long l : proctree.getChips()) {
		Set <Kstat> kschip = proctree.chipStats(l);
		if (showChips) {
		    KstatAggregate ksa = new KstatAggregate(jkstat, kschip);
		    // add this chip to the panel
		    mainPanel.add(new JLabel(chipText + l.toString()));
		    AggregateCpuPanel agp =
				new AggregateCpuPanel(ksa, 1, jkstat);
		    agplist.add(agp);
		    // make wider and thinner than normal
		    agp.setMinimumSize(dchip);
		    agp.setPreferredSize(dchip);
		    mainPanel.add(agp);
		    naggr++;
		}
		if (proctree.isThreaded()) {
		    for (Long ll : proctree.getCores(l)) {
			Set <Kstat> kscore = proctree.coreStats(l, ll);
			if (showCores) {
			    KstatAggregate ksa2 =
					new KstatAggregate(jkstat, kscore);
			    // add this core to the panel
			    mainPanel.add(new JLabel(coreText + ll.toString()));
			    AggregateCpuPanel agp2;
			    agp2 = new AggregateCpuPanel(ksa2, 1, jkstat);
			    agplist.add(agp2);
			    // make wider and thinner than normal
			    agp2.setMinimumSize(dcore);
			    agp2.setPreferredSize(dcore);
			    mainPanel.add(agp2);
			    naggr++;
			}
			if (showThreads) {
			    // show the individual threads in that core
			    for (Kstat ks : kscore) {
				addProcessor(ks);
			    }
			}
		    }
		} else {
		    // not threaded
		    if (showCores) {
			// show the individual cores in that chip
			for (Kstat ks : kschip) {
			    addProcessor(ks);
			}
		    }
		}
	    }
	} else if (proctree.isThreaded()) {
	    // single core cpus
	    // reduce the size of the subcomponents
	    threadheight = midheight;
	    // iterate over chips, showing aggregates over threads
	    for (Long l : proctree.getChips()) {
		Set <Kstat> kschip = proctree.chipStats(l);
		if (showChips) {
		    KstatAggregate ksa = new KstatAggregate(jkstat, kschip);
		    // add this chip to the panel
		    mainPanel.add(new JLabel(chipText + l.toString()));
		    AggregateCpuPanel agp;
		    agp = new AggregateCpuPanel(ksa, 1, jkstat);
		    agplist.add(agp);
		    // make wider and thinner than normal
		    agp.setMinimumSize(dchip);
		    agp.setPreferredSize(dchip);
		    mainPanel.add(agp);
		    naggr++;
		}
		if (showThreads) {
		    // show the individual threads in that chip
		    for (Kstat ks : kschip) {
			addProcessor(ks);
		    }
		}
	    }
	} else {
	    // neither multicore nor multithread
	    // show the individual cpus
	    for (Kstat ks : kstats) {
		addProcessor(ks);
	    }
	}

	SpringUtilities.makeCompactGrid(mainPanel, ncpu+naggr, 2, 6, 3, 3, 3);
    }

    private void repopulate() {
	// stop everything
	stopLoop();
	// clear everything
	ncpu = 0;
	naggr = 0;
	acplist.clear();
	agplist.clear();
	mainPanel.removeAll();
	// and create the main interface again
	populate();
	/*
	 * Now, there's a problem. The first time we select something
	 * the menu sticks on top and everything stops for about 5 seconds.
	 */
	mainPanel.revalidate();
	jf.validate();
	// sanitise the menus
	checkMenus();

	jf.pack();
    }

    private void addMenus() {
	JMenuBar jm = new JMenuBar();

	JMenu jme = new JMenu(svr.getString("FILE.TEXT"));
	jme.setMnemonic(KeyEvent.VK_F);
	if (standalone) {
	    exitItem = new JMenuItem(svr.getString("FILE.EXIT.TEXT"),
				KeyEvent.VK_X);
	} else {
	    exitItem = new JMenuItem(svr.getString("FILE.CLOSEWIN.TEXT"),
				KeyEvent.VK_W);
	}
	exitItem.addActionListener(this);
	jme.add(exitItem);

	jm.add(jme);

	if (proctree.isMulticore() || proctree.isThreaded()) {
	    JMenu jms = new JMenu(svr.getString("SHOW.TEXT"));
	    showChipItem = new JCheckBoxMenuItem(
					svr.getString("CPUSTATE.SHOW.CHIP"));
	    showChipItem.setSelected(showChips);
	    showChipItem.addActionListener(this);
	    jms.add(showChipItem);
	    if (proctree.isMulticore()) {
		showCoreItem = new JCheckBoxMenuItem(
					svr.getString("CPUSTATE.SHOW.CORE"));
		showCoreItem.setSelected(showCores);
		showCoreItem.addActionListener(this);
		jms.add(showCoreItem);
	    }
	    if (proctree.isThreaded()) {
		showThreadItem = new JCheckBoxMenuItem(
					svr.getString("CPUSTATE.SHOW.THREAD"));
		showThreadItem.setSelected(showThreads);
		showThreadItem.addActionListener(this);
		jms.add(showThreadItem);
	    }
	    jm.add(jms);
	}

	checkMenus();

	jf.setJMenuBar(jm);
    }

    /*
     * If the Show menu has only one enabled entry, disable it
     * so that it can't be deselected.
     *
     * This can be dramatically simplified, but the logic would be
     * less clear.
     */
    private void checkMenus() {
	if (proctree.isMulticore() && proctree.isThreaded()) {
	    if ((!showCores) && (!showThreads)) {
		showChipItem.setEnabled(false);
	    } else {
		showChipItem.setEnabled(true);
	    }
	    if ((!showChips) && (!showThreads)) {
		showCoreItem.setEnabled(false);
	    } else {
		showCoreItem.setEnabled(true);
	    }
	    if ((!showChips) && (!showCores)) {
		showThreadItem.setEnabled(false);
	    } else {
		showThreadItem.setEnabled(true);
	    }
	} else if (proctree.isMulticore()) {
	    if (!showCores) {
		showChipItem.setEnabled(false);
	    } else {
		showChipItem.setEnabled(true);
	    }
	    if (!showChips) {
		showCoreItem.setEnabled(false);
	    } else {
		showCoreItem.setEnabled(true);
	    }
	} else if (proctree.isThreaded()) {
	    if (!showThreads) {
		showChipItem.setEnabled(false);
	    } else {
		showChipItem.setEnabled(true);
	    }
	    if (!showChips) {
		showThreadItem.setEnabled(false);
	    } else {
		showThreadItem.setEnabled(true);
	    }
	}
    }

    private void addProcessor(Kstat ks) {
	String scpu = ks.getInstance();
	JLabel jl = new JLabel(cpuText + scpu);
	Font f = jl.getFont();
	jl.setFont(new Font(f.getName(), f.getStyle(), f.getSize()+fontScale));
	mainPanel.add(jl);
	AccessoryCpuPanel acp = new AccessoryCpuPanel("cpu_stat",
				ks.getInst(), "cpu_stat"+scpu, 1, jkstat);
	acplist.add(acp);
	// make wider and thinner than normal
	acp.setMinimumSize(dthread);
	acp.setPreferredSize(dthread);
	mainPanel.add(acp);
	// add a popup menu to each one
	JPopupMenu jpm = new JPopupMenu();
	cpuID[ncpu] = scpu;
	aboutCpuItem[ncpu] = new JMenuItem(cpuLabel + scpu);
	aboutCpuItem[ncpu].addActionListener(this);
	jpm.add(aboutCpuItem[ncpu]);
	extendedCpuItem[ncpu] = new JMenuItem(
					svr.getString("CPUSTATE.EXT.TEXT"));
	extendedCpuItem[ncpu].addActionListener(this);
	jpm.add(extendedCpuItem[ncpu]);
	chartCpuItem[ncpu] = new JMenuItem(cpuChartLabel + scpu);
	chartCpuItem[ncpu].addActionListener(this);
	jpm.add(chartCpuItem[ncpu]);
	PopupListener popupListener = new PopupListener();
	popupListener.setThisPopup(jpm);
	acp.addMouseListener((MouseListener) popupListener);
	ncpu++;
    }

    public void stopLoop() {
	for (AccessoryCpuPanel acp : acplist) {
	    acp.stopLoop();
	}
	for (AggregateCpuPanel agp : agplist) {
	    agp.stopLoop();
	}
    }

    class winExit extends WindowAdapter {
	public void windowClosing(WindowEvent we) {
	    if (standalone) {
		System.exit(0);
	    } else {
		stopLoop();
		jf.dispose();
	    }
	}
    }

    public void actionPerformed(ActionEvent e) {
	if (e.getSource() == exitItem) {
	    if (standalone) {
		System.exit(0);
	    } else {
		stopLoop();
		jf.dispose();
	    }
	}
	if (e.getSource() == showChipItem) {
	    showChips = showChipItem.isSelected();
	    repopulate();
	}
	if (e.getSource() == showCoreItem) {
	    showCores = showCoreItem.isSelected();
	    repopulate();
	}
	if (e.getSource() == showThreadItem) {
	    showThreads = showThreadItem.isSelected();
	    repopulate();
	}
	for (int i = 0; i < ncpus; i++) {
	    if (e.getSource() == aboutCpuItem[i]) {
		new KstatTableFrame("cpu_info", cpuID[i], "cpu_info"+cpuID[i],
				    -1, jkstat);
	    }
	    if (e.getSource() == extendedCpuItem[i]) {
		new KstatTableFrame("cpu_stat", cpuID[i], "cpu_stat"+cpuID[i],
				    1, jkstat);
	    }
	    if (e.getSource() == chartCpuItem[i]) {
		List <String> statlist = new ArrayList <String> ();
		// FIXME what about wait on older versions
		statlist.add("user");
		statlist.add("kernel");
		statlist.add("idle");
		new KstatAreaChartFrame(jkstat,
			new Kstat("cpu_stat", Integer.parseInt(cpuID[i]),
				"cpu_stat"+cpuID[i]),
			statlist, true);
	    }
	}
    }

    /**
     * Inner class to handle mouse popups.
     */
    static class PopupListener extends MouseAdapter {
	public JPopupMenu thisPopup;

	public void setThisPopup(JPopupMenu thisPopup) {
	    this.thisPopup = thisPopup;
	}

	public void mousePressed(MouseEvent e) {
	    maybeShowPopup(e);
	}

	public void mouseReleased(MouseEvent e) {
	    maybeShowPopup(e);
	}

	private void maybeShowPopup(MouseEvent e) {
	    if (e.isPopupTrigger()) {
		thisPopup.show(e.getComponent(), e.getX(), e.getY());
	    }
	}
    }

    // maybe getopts would be nice
    private void parseArgs(String[] args) {
	int i = 0;
	if (args.length > 0) {
	    showChips = false;
	    showCores = false;
	    showThreads = false;
	}
	while (i < args.length) {
	    if (args[i].equals("-p")) {
		showChips = true;
	    }
	    if (proctree.isMulticore() && args[i].equals("-c")) {
		showCores = true;
	    }
	    if (proctree.isThreaded() && args[i].equals("-t")) {
		showThreads = true;
	    }
	    i++;
	}
	// if nothing enabled, then bad args, enable everything
	if (!(showChips || showCores || showThreads)) {
	    showChips = true;
	    showCores = true;
	    showThreads = true;
	}
    }

    public static void main(String[] args) {
	new JCpuState(args);
    }
}
