/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

package uk.co.petertribble.solview.explorer;

import uk.co.petertribble.jkstat.api.*;
import java.util.*;

/**
 * Maintains a description of the processor topology
 */
public class ProcessorTree {

    private JKstat jkstat;

    private Map <Long, Map> map;

    private boolean is_threaded = false;
    private boolean is_multicore = false;

    public ProcessorTree(JKstat jkstat, Set <Kstat> kstats) {
	this.jkstat = jkstat;
	build_tree(kstats);
    }

    public ProcessorTree(JKstat jkstat) {
	this.jkstat = jkstat;
	KstatFilter ksf = new KstatFilter(jkstat);
	ksf.setFilterClass("misc");
	ksf.addFilter("cpu_info:::");
	build_tree(ksf.getKstats());
    }

    /*
     * Walk through the kstats, building up the hierarchy.
     *
     * The top-level Map is a map of <chip-id, Map of cores>
     * The next level map is a map is of <core-id, Map of threads>.
     * And finally, the lowest map is of <thread-id, Kstat>
     *
     * We describe non-threaded cores as cores with a single thread.
     */
    private void build_tree(Set <Kstat> kstats) {
	// TreeMap so we sort
	map = new TreeMap<Long, Map>();

	for (Kstat ks : kstats) {
	    ks = jkstat.getKstat(ks);
	    /*
	     * Build a tree chip_id/core_id/clog_id
	     * Not all chips give us clog_id
	     *
	     * it is assumed that either all chips support clog_id, or none do
	     * our T2 box reports device_ID instead
	     */
	    Long lchip = (Long) ks.getData("chip_id");
	    Long lcore = (Long) ks.getData("core_id");
	    Long lclog = (Long) ks.getData("clog_id");
	    if (lclog == null) {
		lclog = (Long) ks.getData("device_ID");
	    }

	    if (!map.containsKey(lchip)) {
		Map newmap = new TreeMap();
		map.put(lchip, newmap);
	    }
	    Map chipmap = map.get(lchip);

	    /*
	     * If we don't have threads, make up a thread id.
	     *
	     * FIXME: this isn't right. On a T2, the device_ID happens
	     * to match the instance number, going from 0-63.
	     * On the Xeon, clog_id goes from 0-3 on each chip.
	     * In other words, clog_id is only unique within a
	     * physical device. This makes it useless for identifying
	     * a given kstat.
	     * Fortunately, we store the kstat at the bottom and can use
	     * that to get to the real deal. If we are going to use the
	     * clog_id as the key, we need to be careful to not use it
	     * as anything more than a textual identifier.
	     */
	    if (lclog == null) {
		lclog = Long.valueOf(0L);
	    }
	    if (!chipmap.containsKey(lcore)) {
		Map newmap = new TreeMap();
		chipmap.put(lcore, newmap);
	    } else {
		// there was already an entry for this core, so this must be
		// an additional thread in the same core
		is_threaded = true;
	    }
	    Map coremap = (Map) chipmap.get(lcore);
	    coremap.put(lclog, ks);

	    // now test for multicore
	    if (chipmap.size() > 1) {
		is_multicore = true;
	    }
	}
    }

    /**
     * Returns whether the processors are theaded.
     */
    public boolean isThreaded() {
	return is_threaded;
    }

    /**
     * Returns whether the processors have multiple cores.
     */
    public boolean isMulticore() {
	return is_multicore;
    }

    /**
     * Return the number of chips
     */
    public int numChips() {
	/*
	 * There's one entry for each chip in the top map, so the number
	 * of chips is the size of that map.
	 */
	return map.size();
    }

    /**
     * Return the Set of chip ids
     */
    public Set <Long> getChips() {
	return map.keySet();
    }

    /**
     * Return the number of cores in a given chip.
     * If invalid, return zero.
     */
    public int numCores(long chipid) {
	Map m = map.get(Long.valueOf(chipid));
	if (m == null) {
	    return 0;
	}
	return m.size();
    }

    /**
     * Return the Set of core ids for a given chip.
     */
    public Set <Long> getCores(long l) {
	Map m = map.get(Long.valueOf(l));
	if (m == null) {
	    return new HashSet <Long> ();
	}
	Set <Long> set = new HashSet <Long> ();
	for (Object o : m.keySet()) {
	    set.add((Long) o);
	}
	return set;
    }

    /**
     * Return the number of threads in a given core, identified by
     * both chipid and coreid.
     * If invalid, return zero.
     */
    public int numThreads(long chipid, long coreid) {
	Map m = map.get(Long.valueOf(chipid));
	if (m == null) {
	    return 0;
	}
	Map mm = (Map) m.get(coreid);
	if (mm == null) {
	    return 0;
	}
	return mm.size();
    }

    /**
     * Return all the Kstats corresponding to a given chip.
     * If invalid, return the empty Set.
     */
    public Set <Kstat> chipInfoStats(long chipid) {
	Set <Kstat> kss = new HashSet<Kstat>();
	Map m = map.get(Long.valueOf(chipid));
	if (m == null) {
	    return kss;
	}
	for (Object o : m.values()) {
	    Map mm = (Map) o;
	    for (Object oo : mm.values()) {
		Kstat ks = (Kstat) oo;
		kss.add(ks);
	    }
	}
	return kss;
    }

    /**
     * Return all the Kstats corresponding to a given chip.
     * If invalid, return the empty Set.
     */
    public Set <Kstat> chipStats(long chipid) {
	Set <Kstat> kss = new HashSet<Kstat>();
	Map m = map.get(Long.valueOf(chipid));
	if (m == null) {
	    return kss;
	}
	for (Object o : m.values()) {
	    Map mm = (Map) o;
	    for (Object oo : mm.values()) {
		Kstat ks = (Kstat) oo;
		kss.add(new Kstat("cpu_stat", ks.getInst(),
				  "cpu_stat" + ks.getInstance()));
	    }
	}
	return kss;
    }

    /**
     * Return the Kstats for a given core, identified by
     * both chipid and coreid.
     * If invalid, return the empty Set.
     */
    public Set <Kstat> coreInfoStats(long chipid, long coreid) {
	Set <Kstat> kss = new HashSet<Kstat>();
	Map m = map.get(Long.valueOf(chipid));
	if (m == null) {
	    return kss;
	}
	Map mm = (Map) m.get(coreid);
	if (mm == null) {
	    return kss;
	}
	for (Object o : mm.values()) {
	    Kstat ks = (Kstat) o;
	    kss.add(ks);
	}
	return kss;
    }

    /**
     * Return the Kstats for a given core, identified by
     * both chipid and coreid.
     * If invalid, return the empty Set.
     */
    public Set <Kstat> coreStats(long chipid, long coreid) {
	Set <Kstat> kss = new HashSet<Kstat>();
	Map m = map.get(Long.valueOf(chipid));
	if (m == null) {
	    return kss;
	}
	Map mm = (Map) m.get(coreid);
	if (mm == null) {
	    return kss;
	}
	for (Object o : mm.values()) {
	    Kstat ks = (Kstat) o;
	    kss.add(new Kstat("cpu_stat", ks.getInst(),
					"cpu_stat" + ks.getInstance()));
	}
	return kss;
    }

}
